function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]); if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

const _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
      bail = _require$setup.bail,
      info = _require$setup.info,
      error = _require$setup.error;

const _require = require('./utils'),
      getTranslationsMap = _require.getTranslationsMap,
      getNumOfLinks = _require.getNumOfLinks,
      getLocales = _require.getLocales;
/**
 * getLinkErrors
 * Checks for valid markdown link syntax given a JSON file.
 *
 * @param {Array} [{String} filename, {Object} textInFile]
 * @return {Array} errors found
 */


const getLinkErrors = ([filename, textInFile]) => {
  let errors = [];
  let leftSquare = 0,
      rightSquare = 0,
      leftParen = 0,
      rightParen = 0;
  Object.keys(textInFile).forEach(t => {
    const markdown = textInFile[t];

    if (typeof markdown === 'string') {
      // create a mapped array of the number of occurences of each type of
      // bracket in the markdown
      var _map = ['\\[', '\\]', '\\(', '\\)'].map(b => (markdown.match(new RegExp(b, 'g')) || []).length);

      var _map2 = _slicedToArray(_map, 4);

      leftSquare = _map2[0];
      rightSquare = _map2[1];
      leftParen = _map2[2];
      rightParen = _map2[3];

      if (leftSquare !== rightSquare || leftParen !== rightParen) {
        // markdown such as 'A) ...' is accepted
        if (markdown[1] !== ')') {
          errors.push({
            errorName: 'Missing Brackets or Parentheses',
            filename,
            translationKey: t,
            markdown
          });
        }
      } else if ((leftSquare > 0 || rightSquare > 0) && (leftParen > 0 || rightParen > 0)) {
        // there are brackets but the regex can't find any links
        if (getNumOfLinks(markdown) === 0) {
          errors.push({
            errorName: 'Invalid Markdown Link Syntax',
            filename,
            translationKey: t,
            markdown
          });
        }
      }
    }
  });
  return errors;
};
/**
 * showLinkErrors
 * Gets the translations as JSON objects from every file
 * for each locale and calls getLinkErrors. If there are
 * any errors in the file, the error is logged.
 */


const showLinkErrors = () => {
  getLocales().forEach(locale => {
    const translations = getTranslationsMap(locale);

    for (const fileObj of translations) {
      const errors = getLinkErrors(fileObj);

      if (errors.length) {
        errors.forEach(({
          errorName,
          filename,
          translationKey,
          markdown
        }) => {
          error(errorName.toUpperCase());
          info(`${filename} | ${translationKey}`);
          info(`\n${markdown}\n`);
        });
        bail('There are translation errors.');
      }
    }
  });
};

module.exports = {
  showLinkErrors,
  getLinkErrors
};