const axios = require('axios');

const _require = require('./utils'),
      baseUrl = _require.baseUrl,
      authenticate = _require.authenticate;

const _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
      bail = _require$setup.bail,
      success = _require$setup.success,
      info = _require$setup.info;

const findConfig = require('find-config');

const config = findConfig.require('memsource.config.js');

const MEMSOURCE_FILE_IMPORT_SETTINGS = config.MEMSOURCE_FILE_IMPORT_SETTINGS;
/**
 * createFileImportSettings
 * Creates settings based on the MEMSOURCE_FILE_IMPORT_SETINGS key
 *
 */

const createFileImportSettings = async () => {
  // Bail if we can't find file settings config key
  if (typeof MEMSOURCE_FILE_IMPORT_SETTINGS === undefined) {
    bail('Missing MEMSOURCE_FILE_IMPORT_SETTINGS in memsource.config.js');
  }

  const response = await authenticate();
  const token = response.data.token;

  try {
    const settingsResponse = await axios.post(`${baseUrl}importSettings/`, MEMSOURCE_FILE_IMPORT_SETTINGS, {
      params: {
        token
      }
    });
    info(JSON.stringify(settingsResponse.data, null, 2));
    success('Created import settings successfully');
  } catch (e) {
    bail('Failed to create file import settings' + '\n' + e);
  }
};

module.exports = createFileImportSettings;